﻿//
// Copyright (c) 2014, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using MindFusion.DataViews.Wpf;
using System.Collections.ObjectModel;
using MindFusion.DataViews.Wpf.Samples.CS.Model;
using System.Windows.Media.Effects;

namespace MindFusion.DataViews.Wpf.Samples.CS.CustomAppearance
{
	/// <summary>
	/// Interaction logic for Window1.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		private IProjectViewModel project;
		private ObservableCollection<Task> activities;
		private ObservableCollection<Link> dependencies;
		private ObservableCollection<Asset> resources;
		private ObservableCollection<Allocation> allocations;

		private Dictionary<IActivityViewModel, ActivityPresenter> activityPresenters;
		private Dictionary<IResourceAllocationViewModel, AllocationPresenter> allocationPresenters;

		private static readonly DateTime now = DateTime.Now;
		private static readonly Random rnd = new Random(now.Millisecond);

		public MainWindow()
		{
			InitializeComponent();

			activities = new ObservableCollection<Task>();
			dependencies = new ObservableCollection<Link>();
			resources = new ObservableCollection<Asset>();
			allocations = new ObservableCollection<Allocation>();

			activityPresenters = new Dictionary<IActivityViewModel, ActivityPresenter>();
			allocationPresenters = new Dictionary<IResourceAllocationViewModel, AllocationPresenter>();

			// Create the project and assign it to the activity view.
			project = activityView.CreateViewModel(activities, dependencies, resources, allocations,
				typeof(Task), typeof(Link), typeof(Asset), typeof(Allocation));

			activityView.ActivityClicked += activityView_ActivityClicked;
			activityView.ActivityModified += activityView_ActivityModified;
			activityView.ActivityModifying += new EventHandler<ProjectValidationEventArgs>(activityView_ActivityModifying);

			// Generate some data for the project.
			GenerateTasks(20);
			var subtask = project.CreateActivity(project.Activities.ElementAt(0), -1);
			subtask.ActivityName = "Task 1.1";
			subtask.StartTime = subtask.Parent.StartTime;
			subtask.Duration = TimeSpan.FromDays(2);
			subtask.Progress = 0.6;
			subtask.ScheduleMode = ScheduleMode.SemiAuto;
			subtask.Parent.ScheduleMode = ScheduleMode.Auto;
			project.Activities.ElementAt(3).IsMilestone = true;
			GenerateLinks(10);
			GenerateResources(5);
			GenerateAllocations(10);

			activityView.ActivityClicked += (s, e) =>
				{
					if (e.Element is GridRow)
					{
						var el = (GridRow)e.Element;
						var activity = el.DataItem as IActivityViewModel;

						if (e.RightButtonDown)
						{
							var menu = new ContextMenu();
							var item = new MenuItem { Header = "item 1" };
							item.Click += (s1, e1) =>
								{
									MessageBox.Show("item 1 clicked for " + activity.ActivityName);
								};
							menu.Items.Add(item);
							item = new MenuItem { Header = "item 2" };
							menu.Items.Add(item);
							menu.Placement = System.Windows.Controls.Primitives.PlacementMode.Mouse;
							menu.IsOpen = true;
						}
					}
					else if (e.Element is ActivityPresenter)
					{
						MessageBox.Show("activity clicked");
					}
				};

			activityView.Loaded += (s, e) =>
				{
					for (var i = 0; i < activityView.Columns.Count; i++)
					{
						activityView.Columns[i].Width = i * 10;
					}
				};

			activityView.HeaderBackground = new SolidColorBrush(Color.FromArgb(0xFF, 0x6A, 0x52, 0x8C));
			activityView.HeaderForeground = Brushes.White;
			activityView.HeaderMargin = new Thickness(4);
			activityView.HeaderBorderBrush = Brushes.White;
			activityView.HeaderBorderThickness = new Thickness(0, 0, 1, 1);
		}

		void activityView_ActivityModifying(object sender, ProjectValidationEventArgs e)
		{
			UpdateAllocationCollisions();
		}

		private void UpdateAllocationCollisions()
		{
			foreach (var presenter in allocationPresenters.Values)
			{
				if (presenter.IsInCollision)
				{
					presenter.BorderBrush = Brushes.Red;
					presenter.AccentBrush = Brushes.Red;
					presenter.MouseOverBorderBrush = Brushes.Red;
				}
			}
		}

		void activityView_ActivityModified(object sender, ProjectEventArgs e)
		{

		}

		void activityView_ActivityClicked(object sender, ProjectMouseEventArgs e)
		{
			var presenter = e.Element as ActivityPresenter;
			if (presenter == null) return;
		}

		/// <summary>
		/// Generates a list with task objects with random properties.
		/// </summary>
		/// <param name="count">
		/// number of tasks to create.
		/// </param>
		private void GenerateTasks(int count)
		{
			for (int i = 0; i < count; i++)
			{
				var activity = project.CreateActivity(null, -1);
				activity.ActivityName = "Task " + (i + 1);
				activity.StartTime = now.AddDays(rnd.Next(0, 15));
				activity.EndTime = activity.StartTime.AddHours(rnd.Next(3, 72));
				activity.Progress = rnd.NextDouble();
			}
		}

		/// <summary>
		/// Generates some random links between tasks.
		/// </summary>
		/// <param name="count">
		/// number of links to create.
		/// </param>
		private void GenerateLinks(int count)
		{
			var coll = new List<IActivityViewModel>(project.Activities);
			var len = coll.Count - 1;
			for (int i = 0; i < count; i++)
			{
				var dependency = project.CreateDependency(coll[rnd.Next(1, len)], coll[rnd.Next(1, len)]);
				if (dependency != null)
					dependency.DependencyType = (DependencyType)rnd.Next(0, 3);
			}
		}

		/// <summary>
		/// Generate some resources.
		/// </summary>
		/// <param name="count">
		/// number of resources to create.
		/// </param>
		private void GenerateResources(int count)
		{
			for (int i = 0; i < count; i++)
			{
				var resource = project.CreateResource();
				resource.Title = "Resource " + (i + 1);
				resource.Description = "Description for " + resource.Title;
			}
		}

		/// <summary>
		/// Generate some allocations.
		/// </summary>
		/// <param name="count">
		/// number of allocations to create.
		/// </param>
		private void GenerateAllocations(int count)
		{
			for (int i = 0; i < count; i++)
			{
				var task = project.Activities.ElementAt(rnd.Next(0, project.Activities.Count() - 1));
				var asset = project.Resources.ElementAt(rnd.Next(0, project.Resources.Count() - 1));
				project.CreateResourceAllocation(asset, task);
			}
		}

		/// <summary>
		/// Change the theme.
		/// </summary>
		private void themeBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			var cmb = sender as ComboBox;
			if (cmb == null || !cmb.IsLoaded) return;
			if (cmb.SelectedItem == null) return;
			var content = (cmb.SelectedItem as ComboBoxItem).Content.ToString();
			LayoutRoot.Resources.MergedDictionaries.Clear();
			viewContainer.Resources.MergedDictionaries.Clear();
			allocationPresenters.Clear();
			activityPresenters.Clear();
			var res = new ResourceDictionary();

			res.MergedDictionaries.Add(
				new ResourceDictionary()
				{
					Source = new Uri(String.Format("/MindFusion.DataViews.Wpf;component/Themes/{0}.xaml", content), UriKind.RelativeOrAbsolute)
				});

			if (themingStyle.IsChecked.Value)
				LayoutRoot.Resources = res;
			else
				viewContainer.Resources = res;
		}

		/// <summary>
		/// Handle the ActivityPresenterCreated event to give the presenter a customized look.
		/// </summary>
		private void activityView_ActivityPresenterCreated(object sender, BasePresenterEventArgs e)
		{
			var activityPresenter = e.Presenter as ActivityPresenter;
			if (activityPresenter == null) return;
			var vm = activityPresenter.DataContext as IActivityViewModel;
			if (vm != null)
			{
				if (!activityPresenters.Keys.Contains(vm))
					activityPresenters.Add(vm, activityPresenter);
			}

			activityPresenter.CornerRadius = 2;
			activityPresenter.ResourcesLabelFontStyle = FontStyles.Italic;
			activityPresenter.ResourcesLabelFontStretch = FontStretches.Condensed;
			activityPresenter.ResourcesLabelForeground = Brushes.DarkGreen;
		}

		/// <summary>
		/// Handle the DependencyPresenterCreated event to give the presenter a customized look.
		/// </summary>
		private void activityView_DependencyPresenterCreated(object sender, DependencyPresenterEventArgs e)
		{
			var dependencyPresneter = e.Presenter;
			if (dependencyPresneter == null) return;

			if (dependencyPresneter.DependencyType == DependencyType.StartToStart)
			{
				dependencyPresneter.Background = Brushes.Red;
				dependencyPresneter.BorderBrush = Brushes.Red;
			}
		}

		/// <summary>
		/// Apply some premade customizations to items in the charts.
		/// </summary>
		private void CheckBox_Checked(object sender, RoutedEventArgs e)
		{
			var src = e.Source as CheckBox;
			if (src == null) return;

			string propertyName = src.Content.ToString();

			switch (propertyName)
			{
				case "ItemAccentBrush":
					activityView.ItemAccentBrush = Brushes.Black;
					break;
				case "ItemBackground":
					activityView.ItemBackground = new LinearGradientBrush(
						new GradientStopCollection(3)
						{
							new GradientStop(Color.FromRgb(0x00, 0x7F, 0x0E),0.0),
							new GradientStop(Color.FromRgb(0x7B, 0xFF, 0x23),0.5),
							new GradientStop(Color.FromRgb(0x00, 0x7F, 0x0E),1.0)
						}, 90);
					break;
				case "ItemBorderBrush":
					activityView.ItemBorderBrush = Brushes.DarkGreen;
					break;
				case "ItemFocusBrush":
					activityView.ItemFocusBrush = Brushes.White;
					break;
				case "ItemMouseOverBackground":
					activityView.ItemMouseOverBackground = new LinearGradientBrush(
						new GradientStopCollection(3)
						{
							new GradientStop(Color.FromRgb(0x7B, 0xFF, 0x23),0.0),
							new GradientStop(Color.FromRgb(0x00, 0x7F, 0x0E),0.5),
							new GradientStop(Color.FromRgb(0x7B, 0xFF, 0x23),1.0)
						}, 90);
					break;
				case "ItemMouseOverBorderBrush":
					activityView.ItemMouseOverBorderBrush = Brushes.Green;
					break;
				case "DependenciesArrowSize":
					activityView.DependenciesArrowSize = 3;
					break;
				case "DependenciesCornerRadius":
					activityView.DependenciesCornerRadius = 6;
					break;
				case "DependenciesFill":
					activityView.DependenciesFill = Brushes.Orange;
					break;
				case "DependenciesStroke":
					activityView.DependenciesStroke = Brushes.Orange;
					break;
				case "DependenciesStrokeThickness":
					activityView.DependenciesStrokeThickness = 2;
					break;
				default:
					break;
			}
		}

		/// <summary>
		/// Clear the customizations of the items in the chart.
		/// </summary>
		private void CheckBox_Unchecked(object sender, RoutedEventArgs e)
		{
			var src = e.Source as CheckBox;
			if (src == null) return;
			string propertyName = src.Content.ToString();

			switch (propertyName)
			{
				case "ItemAccentBrush":
					activityView.ClearValue(BaseChart.ItemAccentBrushProperty);
					break;
				case "ItemBackground":
					activityView.ClearValue(BaseChart.ItemBackgroundProperty);
					break;
				case "ItemBorderBrush":
					activityView.ClearValue(BaseChart.ItemBorderBrushProperty);
					break;
				case "ItemFocusBrush":
					activityView.ClearValue(BaseChart.ItemFocusBrushProperty);
					break;
				case "ItemMouseOverBackground":
					activityView.ClearValue(BaseChart.ItemMouseOverBackgroundProperty);
					break;
				case "ItemMouseOverBorderBrush":
					activityView.ClearValue(BaseChart.ItemMouseOverBorderBrushProperty);
					break;
				case "DependenciesArrowSize":
					activityView.ClearValue(ActivityChart.DependenciesArrowSizeProperty);
					break;
				case "DependenciesCornerRadius":
					activityView.ClearValue(ActivityChart.DependenciesCornerRadiusProperty);
					break;
				case "DependenciesFill":
					activityView.ClearValue(ActivityChart.DependenciesFillProperty);
					break;
				case "DependenciesStroke":
					activityView.ClearValue(ActivityChart.DependenciesStrokeProperty);
					break;
				case "DependenciesStrokeThickness":
					activityView.ClearValue(ActivityChart.DependenciesStrokeThicknessProperty);
					break;
				default:
					break;
			}
		}

		private void button1_Click(object sender, RoutedEventArgs e)
		{
			var ts = new TimelineSettings();
			ts.Unit = TimeUnit.Hour;
			ts.UnitCount = 12;
			ts.Format = "hh";
//			activityView.allowdr
			activityView.DrillTo(DateTime.Today, ts);
		}
	}
}
